<?php

/**
 * @file
 * This file defines the default fields to be added
 * to campaigns and supporters.
 */

/**
 * Returns a structured array defining the fields created by the default content type.
 *
 * This is factored into this function so it can be used in both
 * citcon_install() and citcon_uninstall().
 *
 * @return
 *  An associative array specifying the fields we wish to add to our
 *  new node type.
 */
function _citcon_installed_campaign_fields() {
  // During installation, the t() function is unavailable, so we use get_t()
  // to store the name of the translation function.
  $t = get_t();

  // Initialize empty fields array
  $fields = array();

  //***********************
  // "Settings" page fields
  //***********************
  // "Campaign Status" fields
  $fields['citcon_campaign_status'] = array(
    'field_name' => 'citcon_campaign_status',
    'cardinality' => 1,
    'type' => 'list_integer',
    'settings' => array(
      'citcon shared setting' => TRUE,
      'citcon admin group' => 'settings',
      'citcon fieldsets' => '',
      'allowed_values' => array(
        CITCON_CAMPAIGN_ACTIVE => $t('Active'),
        CITCON_CAMPAIGN_TEST => $t('Test Mode'),
        CITCON_CAMPAIGN_INACTIVE => $t('Inactive'),
      ),
    ),
  );

  // "Goal" fields
  $fields['citcon_goal'] = array(
    'field_name' => 'citcon_goal',
    'cardinality' => 1,
    'type' => 'number_integer',
    'settings' => array(
      'citcon shared setting' => TRUE,
      'citcon admin group' => 'settings',
      'citcon fieldsets' => 'Active Options][Goal',
    ),
  );
  $fields['citcon_goal_offset'] = array(
    'field_name' => 'citcon_goal_offset',
    'cardinality' => 1,
    'type' => 'number_integer',
    'settings' => array(
      'citcon shared setting' => TRUE,
      'citcon admin group' => 'settings',
      'citcon fieldsets' => 'Active Options][Goal',
    ),
  );
  // "Goal Plugin" fields... these will always be hidden.
  $fields['citcon_plugin_goal'] = array(
    'field_name' => 'citcon_plugin_goal',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'settings',
      'citcon hide field' => TRUE,
    ),
  );
  // "Settings Plugin" fields... these will always be hidden.
  $fields['citcon_plugin_settings'] = array(
    'field_name' => 'citcon_plugin_settings',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'settings',
      'citcon hide field' => TRUE,
    ),
  );

  // "Test Email Addresses" fields
  $fields['citcon_test_emails'] = array(
    'field_name' => 'citcon_test_emails',
    'cardinality' => FIELD_CARDINALITY_UNLIMITED,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'settings',
      'citcon fieldsets' => 'Test Options',
    ),
  );
  $fields['citcon_test_opt'] = array(
    'field_name' => 'citcon_test_opt',
    'cardinality' => 1,
    'type' => 'list_integer',
    'settings' => array(
      'citcon admin group' => 'settings',
      'citcon fieldsets' => 'Test Options',
      'allowed_values' => array(
        CITCON_TEST_DISPLAY => $t("No emails will be sent, but debug information will be displayed."),
        CITCON_TEST_EMAIL_DEBUG => $t('Send emails to "Test Email Addresses" with debug information at the top.'),
        CITCON_TEST_EMAIL => $t('Send emails to "Test Email Addresses" without debug information. Just as the end user would see it.'),
      ),
    ),
  );

  // "Inactive" fields
  $fields['citcon_inactive_message'] = array(
    'field_name' => 'citcon_inactive_message',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'settings',
      'citcon fieldsets' => 'Inactive Options',
      'citcon token support' => array(
        '#citcon_campaign_token_support',
      ),
    ),
  );

  // "Sharing Plugin" fields... these will always be hidden.
  $fields['citcon_plugin_sharing'] = array(
    'field_name' => 'citcon_plugin_sharing',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'settings',
      'citcon hide field' => TRUE,
    ),
  );

  //***********************
  // "Petition" fields
  //***********************
  $fields['citcon_pet_dest_path'] = array(
    'field_name' => 'citcon_pet_dest_path',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'petition',
    ),
  );
  $fields['citcon_pet_dest_title'] = array(
    'field_name' => 'citcon_pet_dest_title',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'petition',
      'citcon token support' => array(
        '#citcon_campaign_token_support',
        '#citcon_no_html_token_support',
      ),
    ),
  );
  $fields['citcon_pet_dest_text'] = array(
    'field_name' => 'citcon_pet_dest_text',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'petition',
      'citcon token support' => array(
        '#citcon_campaign_token_support',
      ),
    ),
  );

  // "Primary" list fields
  $fields['citcon_petition_pri_rows'] = array(
    'field_name' => 'citcon_petition_pri_rows',
    'cardinality' => 1,
    'type' => 'number_integer',
    'settings' => array(
      'citcon shared setting' => TRUE,
      'citcon admin group' => 'petition',
      'citcon fieldsets' => 'Primary Petition',
    ),
  );
  $fields['citcon_petition_pri_text'] = array(
    'field_name' => 'citcon_petition_pri_text',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'petition',
      'citcon fieldsets' => 'Primary Petition',
      'citcon token support' => array(
        '#citcon_supporter_token_support',
        '#citcon_campaign_token_support',
      ),
    ),
  );

  // "Secondary" list fields
  $fields['citcon_petition_sec_rows'] = array(
    'field_name' => 'citcon_petition_sec_rows',
    'cardinality' => 1,
    'type' => 'number_integer',
    'settings' => array(
      'citcon admin group' => 'petition',
      'citcon fieldsets' => 'Secondary Petition',
    ),
  );
  $fields['citcon_petition_sec_text'] = array(
    'field_name' => 'citcon_petition_sec_text',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'petition',
      'citcon fieldsets' => 'Secondary Petition',
      'citcon token support' => array(
        '#citcon_supporter_token_support',
        '#citcon_campaign_token_support',
      ),
    ),
  );
  // "Petition Plugin" fields... these will always be hidden.
  $fields['citcon_plugin_petition'] = array(
    'field_name' => 'citcon_plugin_petition',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'petition',
      'citcon hide field' => TRUE,
    ),
  );

  //***********************
  // "Fields" page fields
  //***********************
  // "Supporter Type" fields
  $fields['citcon_fields_supporter_type'] = array(
    'field_name' => 'citcon_fields_supporter_type',
    'cardinality' => 1,
    'type' => 'list_text',
    'settings' => array(
      //'citcon shared setting' => TRUE,
      'citcon admin group' => 'fields',
      'citcon fieldsets' => '',
      'allowed_values_function' => 'citcon_get_supporter_type_options',
    ),
  );

  // "Opt Out" fields
  $fields['citcon_petition_opt_field'] = array(
    'field_name' => 'citcon_petition_opt_field',
    'cardinality' => 1,
    'type' => 'list_text',
    'settings' => array(
      //'citcon shared setting' => TRUE,
      'citcon admin group' => 'fields',
      'citcon fieldsets' => 'Field Mapping',
      'allowed_values_function' => 'citcon_get_checkbox_field_options',
    ),
  );

  // "Email Field" fields
  $fields['citcon_supporter_email_field'] = array(
    'field_name' => 'citcon_supporter_email_field',
    'cardinality' => 1,
    'type' => 'list_text',
    'settings' => array(
      //'citcon shared setting' => TRUE,
      'citcon admin group' => 'fields',
      'citcon fieldsets' => 'Field Mapping',
      'allowed_values_function' => 'citcon_get_supporter_email_field_options',
    ),
  );
  $fields['citcon_lookup_supporter'] = array(
    'field_name' => 'citcon_lookup_supporter',
    'cardinality' => 1,
    'type' => 'list_integer',
    'settings' => array(
      //'citcon shared setting' => TRUE,
      'citcon admin group' => 'fields',
      'allowed_values' => array(
        CITCON_CHECKBOX_OFF => $t('Do not display Avaaz Style Returning Supporter Lookup'),
        CITCON_CHECKBOX_ON => $t('Display Avaaz Style Returning Supporter Lookup'),
      ),
    ),
  );

  // Fieldset options
  $fields['citcon_slf_title'] = array(
    'field_name' => 'citcon_slf_title',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'fields',
      'citcon fieldsets' => 'Form Groups][Supporter Lookup Fieldset',
    ),
  );
  $fields['citcon_slf_desc'] = array(
    'field_name' => 'citcon_slf_desc',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'fields',
      'citcon fieldsets' => 'Form Groups][Supporter Lookup Fieldset',
      'citcon token support' => array(
        '#citcon_campaign_token_support',
      ),
    ),
  );
  $fields['citcon_pif_title'] = array(
    'field_name' => 'citcon_pif_title',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'fields',
      'citcon fieldsets' => 'Form Groups][Personal Info Fieldset',
    ),
  );
  $fields['citcon_pif_desc'] = array(
    'field_name' => 'citcon_pif_desc',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'fields',
      'citcon fieldsets' => 'Form Groups][Personal Info Fieldset',
      'citcon token support' => array(
        '#citcon_campaign_token_support',
      ),
    ),
  );

  // "Form Labels" fields
  $fields['citcon_fields_submit_text'] = array(
    'field_name' => 'citcon_fields_submit_text',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'fields',
      'citcon fieldsets' => 'Display',
    ),
  );
  // "From label" fields... this will always be hidden.
  $fields['citcon_field_labels'] = array(
    'field_name' => 'citcon_field_labels',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'fields',
      'citcon hide field' => TRUE,
    ),
  );

  // "Sharing Plugin" fields... these will always be hidden.
  $fields['citcon_plugin_fields'] = array(
    'field_name' => 'citcon_plugin_fields',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'fields',
      'citcon hide field' => TRUE,
    ),
  );



  //***********************
  // "After Sign Up" page fields
  //***********************
  // "Destination" fields
  $fields['citcon_destination'] = array(
    'field_name' => 'citcon_destination',
    'cardinality' => 1,
    'type' => 'list_integer',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Destination',
      'allowed_values' => array(
        CITCON_DESTINATION_NONE => $t('No end destination. Just reload the campaign page but display a confirmation message.'),
        CITCON_DESTINATION_EXTERNAL => $t('Redirect to an external URL.'),
        CITCON_DESTINATION_INTERNAL => $t('Redirect to a special campaign "Thank You" page where you have campaign specific options, like tokens and share with a friend functionality.'),
      ),
    ),
  );
  $fields['citcon_dest_ext_url'] = array(
    'field_name' => 'citcon_dest_ext_url',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Destination',
    ),
  );
  $fields['citcon_dest_msg'] = array(
    'field_name' => 'citcon_dest_msg',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Destination',
      'citcon token support' => array(
        '#citcon_supporter_token_support',
        '#citcon_campaign_token_support',
        '#citcon_lookup_token_support',
        '#citcon_no_html_token_support',
      ),
    ),
  );
  $fields['citcon_thank_dest_path'] = array(
    'field_name' => 'citcon_thank_dest_path',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Destination',
    ),
  );
  $fields['citcon_thank_dest_title'] = array(
    'field_name' => 'citcon_thank_dest_title',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Destination',
      'citcon token support' => array(
        '#citcon_supporter_token_support',
        '#citcon_campaign_token_support',
        '#citcon_lookup_token_support',
        '#citcon_no_html_token_support',
      ),
    ),
  );
  $fields['citcon_thank_dest_text'] = array(
    'field_name' => 'citcon_thank_dest_text',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Destination',
      'citcon token support' => array(
        '#citcon_supporter_token_support',
        '#citcon_campaign_token_support',
        '#citcon_lookup_token_support',
      ),
    ),
  );
  // "Thank You Email" fields
  $fields['citcon_thank_email_status'] = array(
    'field_name' => 'citcon_thank_email_status',
    'cardinality' => 1,
    'type' => 'list_integer',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Thank You Email',
      'allowed_values' => array(
        CITCON_CHECKBOX_OFF => $t('Do not send a thank you email'),
        CITCON_CHECKBOX_ON => $t('Send a "Thank You" Email'),
      ),
    ),
  );
  $fields['citcon_thank_email_subject'] = array(
    'field_name' => 'citcon_thank_email_subject',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Thank You Email',
      'citcon token support' => array(
        '#citcon_supporter_token_support',
        '#citcon_campaign_token_support',
        '#citcon_lookup_token_support',
        '#citcon_no_html_token_support',
      ),
    ),
  );
  $fields['citcon_thank_email_body'] = array(
    'field_name' => 'citcon_thank_email_body',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Thank You Email',
      'citcon token support' => array(
        '#citcon_supporter_token_support',
        '#citcon_campaign_token_support',
        '#citcon_lookup_token_support',
        '#citcon_no_html_token_support',
      ),
    ),
  );

  // "Data Processing" fields
  $fields['citcon_data_save_to_drupal'] = array(
    'field_name' => 'citcon_data_save_to_drupal',
    'cardinality' => 1,
    'type' => 'list_integer',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => 'Data Processing',
      'allowed_values' => array(
        CITCON_CHECKBOX_OFF => $t('Do not save supporter data'),
        CITCON_CHECKBOX_ON => $t('Save Supporter Data to Drupal Database'),
      ),
    ),
  );

  // "Data Export Plugin" fields... these will always be hidden.
  $fields['citcon_plugin_data_process'] = array(
    'field_name' => 'citcon_plugin_data_process',
    'cardinality' => 1,
    'type' => 'text_long',
    'settings' => array(
      'citcon admin group' => 'after',
      'citcon fieldsets' => '',
      'citcon hide field' => TRUE,
    ),
  );

  return $fields;
}

/**
 * Returns a structured array defining the instances for this content type.
 *
 * This is factored into this function so it can be used in both
 * node_example_install() and node_example_uninstall().
 *
 * @return
 *  An associative array specifying the instances we wish to add to our new
 *  node type.
 */
function _citcon_installed_campaign_instances() {
  $t = get_t();

  // Initialize empty instances array
  $insts = array();

  //***********************
  // "Settings" page instances
  //***********************
  // "Campaign Status" instances
  $insts['citcon_campaign_status'] = array(
    'field_name' => 'citcon_campaign_status',
    'label' => $t('Campaign Status'),
    'description' => $t('This is the campaign status.'),
    'widget' => array(
      'type' => 'options_buttons',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_CAMPAIGN_ACTIVE,
      ),
    ),
    'required' => TRUE,
  );

  // "Goal" instances
  $insts['citcon_goal'] = array(
    'field_name' => 'citcon_goal',
    'label' => $t('Goal'),
    'widget' => array(
      'type' => 'number',
    ),
    'default_value' => array(
      array(
        'value' => 5000,
      ),
    ),
  );
  $insts['citcon_goal_offset'] = array(
    'field_name' => 'citcon_goal_offset',
    'label' => $t('Goal Offset'),
    'description' => $t('In case this campaign already has supporters, you can offset the count here.'),
    'widget' => array(
      'type' => 'number',
    ),
    'default_value' => array(
      array(
        'value' => 0,
      ),
    ),
  );
  // "Goal Plugin" instances... these will always be hidden.
  $insts['citcon_plugin_goal'] = array(
    'field_name' => 'citcon_plugin_goal',
    'label' => $t('Goal Plugin'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );
  // "Settings Plugin" instances... these will always be hidden.
  $insts['citcon_plugin_settings'] = array(
    'field_name' => 'citcon_plugin_settings',
    'label' => $t('Settings Plugin'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  // "Test Email Addresses" instances
  $insts['citcon_test_emails'] = array(
    'field_name' => 'citcon_test_emails',
    'label' => $t('Test Email Addresses'),
    'description' => $t('While your campaign is in Test mode, emails will be ONLY be sent to these email addresses.'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
  );
  $insts['citcon_test_opt'] = array(
    'field_name' => 'citcon_test_opt',
    'label' => $t('Test mode behaviour'),
    'description' => $t('What should happen to emails in test mode?'),
    'widget' => array(
      'type' => 'options_buttons',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_TEST_DISPLAY,
      ),
    ),
    'required' => TRUE,
  );

  // "Inactive" instances
  $insts['citcon_inactive_message'] = array(
    'field_name' => 'citcon_inactive_message',
    'label' => $t('Inactive Message'),
    'description' => $t('While your campaign is Inactive, visitors will be see this.'),
    'settings' => array(
      'text_processing' => 1,
    ),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  // "Sharing Plugin" instances... these will always be hidden.
  $insts['citcon_plugin_sharing'] = array(
    'field_name' => 'citcon_plugin_sharing',
    'label' => $t('Sharing Plugin'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  //***********************
  // "Petition" page instances
  //***********************
  // Petition page
  $insts['citcon_pet_dest_path'] = array(
    'field_name' => 'citcon_pet_dest_path',
    'label' => $t('Petition Page - Path'),
    'description' => $t('The URL path of the Petition page.'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'default_value' => array(
      array(
        'value' => 'petition',
      ),
    ),
    'required' => TRUE,
  );
  $insts['citcon_pet_dest_title'] = array(
    'field_name' => 'citcon_pet_dest_title',
    'label' => $t('Petition Page - Title'),
    'description' => $t('The title of the Petition page.'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
  );
  $insts['citcon_pet_dest_text'] = array(
    'field_name' => 'citcon_pet_dest_text',
    'label' => $t('Petition Page - Body'),
    'description' => $t('Text to display on the petition page.'),
    'settings' => array(
      'text_processing' => 1,
    ),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  // "Primary" list instances
  $insts['citcon_petition_pri_rows'] = array(
    'field_name' => 'citcon_petition_pri_rows',
    'label' => $t('Number of Rows to Display'),
    'description' => $t('Number of rows to display before the pagination kicks in.'),
    'widget' => array(
      'type' => 'number',
    ),
    'default_value' => array(
      array(
        'value' => 5,
      ),
    ),
  );
  $insts['citcon_petition_pri_text'] = array(
    'field_name' => 'citcon_petition_pri_text',
    'label' => $t('Petition Row Display'),
    'description' => $t('Here you can customize how each petition row will appear.'),
    'settings' => array(
      'text_processing' => 1,
    ),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  // "Secondary" list instances
  $insts['citcon_petition_sec_rows'] = array(
    'field_name' => 'citcon_petition_sec_rows',
    'label' => $t('Number of Rows to Display'),
    'description' => $t('Number of rows to display before the pagination kicks in.'),
    'widget' => array(
      'type' => 'number',
    ),
    'default_value' => array(
      array(
        'value' => 5,
      ),
    ),
  );
  $insts['citcon_petition_sec_text'] = array(
    'field_name' => 'citcon_petition_sec_text',
    'label' => $t('Petition Row Display'),
    'description' => $t('Here you can customize how each petition row will appear.'),
    'settings' => array(
      'text_processing' => 1,
    ),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );
  // "Petition Plugin" instances... these will always be hidden.
  $insts['citcon_plugin_petition'] = array(
    'field_name' => 'citcon_plugin_petition',
    'label' => $t('Petition Plugin'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  //***********************
  // "Fields" page instances
  //***********************
  // "Supporter Type" instances
  $insts['citcon_fields_supporter_type'] = array(
    'field_name' => 'citcon_fields_supporter_type',
    'label' => $t('Supporter Type'),
    'description' => $t('Which supporter should be used. A.K.A. Which form fields will we show to the supporter.'),
    'widget' => array(
      'type' => 'options_select',
    ),
    //'default_value_function' => 'citcon_get_default_supporter_type',
    'default_value' => array(
      array(
        'value' => 'email_standard_canada',
      ),
    ),
  );

  // "Opt Out" instances
  $insts['citcon_petition_opt_field'] = array(
    'field_name' => 'citcon_petition_opt_field',
    'label' => $t('Petition Opt Out Field'),
    'description' => $t('Which field should we use to check for "Opt in to display my info in the petition".'),
    'widget' => array(
      'type' => 'options_select',
    ),
    'default_value' => array(
      array(
        'value' => 'field_citcon_pet_opt_in',
      ),
    ),
  );

  // "Email Field" instances
  $insts['citcon_supporter_email_field'] = array(
    'field_name' => 'citcon_supporter_email_field',
    'label' => $t('Supporter Email Field'),
    'description' => $t("CitCon needs to know which field is the supporter's email field so that we can do things like send thank you emails and such."),
    'widget' => array(
      'type' => 'options_select',
    ),
    'default_value' => array(
      array(
        'value' => 'field_citcon_email',
      ),
    ),
  );
  $insts['citcon_lookup_supporter'] = array(
    'field_name' => 'citcon_lookup_supporter',
    'label' => $t('Display Avaaz Style Returning Supporter Lookup'),
    'description' => $t('Checking this option will create a "Been here before? Enter your email address" field. Which will lookup the email address and use (but not display) any previously gathered information.'),
    'widget' => array(
      'type' => 'options_onoff',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_CHECKBOX_OFF,
      ),
    ),
  );

  // Fieldset options
  $insts['citcon_slf_title'] = array(
    'field_name' => 'citcon_slf_title',
    'label' => $t('Supporter Lookup Group Title'),
    'description' => $t('This will be the title for the Supporter Lookup group.'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'default_value' => array(
      array(
        'value' => 'Been here before?',
      ),
    ),
  );
  $insts['citcon_slf_desc'] = array(
    'field_name' => 'citcon_slf_desc',
    'label' => $t('Supporter Lookup Group Helper Text'),
    'description' => $t('This text will be displayed in the Supporter Lookup group.'),
    'settings' => array(
      'text_processing' => 1,
    ),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );
  $insts['citcon_pif_title'] = array(
    'field_name' => 'citcon_pif_title',
    'label' => $t('Personal Information Group Title'),
    'description' => $t('This will be the title for the Personal Information group.'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'default_value' => array(
      array(
        'value' => 'Sign your Message',
      ),
    ),
  );
  $insts['citcon_pif_desc'] = array(
    'field_name' => 'citcon_pif_desc',
    'label' => $t('Personal Information Group Helper Text'),
    'description' => $t('This text will be displayed in the Personal Information group. You may want to put something like a privacy policy here.'),
    'settings' => array(
      'text_processing' => 1,
    ),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  // "Form Labels" instances
  $insts['citcon_fields_submit_text'] = array(
    'field_name' => 'citcon_fields_submit_text',
    'label' => $t('Submit Button Text'),
    'description' => $t('The text to be displayed on the form submit button. <em>EG. "Send Email", or "Sign Petition", or "Show us your support", or use the boring default... "Submit"</em>'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'default_value' => array(
      array(
        'value' => $t('Submit'),
      ),
    ),
    'required' => TRUE,
  );
  // "Sharing Plugin" instances... these will always be hidden.
  $insts['citcon_field_labels'] = array(
    'field_name' => 'citcon_field_labels',
    'label' => $t('Field Label Settings'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  // "Field Plugin" instances... these will always be hidden.
  $insts['citcon_plugin_fields'] = array(
    'field_name' => 'citcon_plugin_fields',
    'label' => $t('Fields Plugin'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );


  //***********************
  // "After Sign Up" page instances
  //***********************
  // "Opt Out" instances
  $insts['citcon_destination'] = array(
    'field_name' => 'citcon_destination',
    'label' => $t('Supporter Destination Option'),
    'description' => $t('Where should we send your supporter after they sign the petition?'),
    'widget' => array(
      'type' => 'options_buttons',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_DESTINATION_NONE,
      ),
    ),
    'required' => TRUE,
  );

  // "External" options instances
  $insts['citcon_dest_ext_url'] = array(
    'field_name' => 'citcon_dest_ext_url',
    'label' => $t('External URL'),
    'description' => $t('The external URL to redirect to.<br /> For example: <em>http://google.com</em>'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
  );

  // "Internal" list instances
  $insts['citcon_dest_msg'] = array(
    'field_name' => 'citcon_dest_msg',
    'label' => $t('Confirmation Message'),
    'description' => $t('This status message appears at the top of the page.'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
    'default_value' => array(
      array(
        'value' => $t('Thank you for supporting this campaign.'),
      ),
    ),
    'required' => TRUE,
  );
  $insts['citcon_thank_dest_path'] = array(
    'field_name' => 'citcon_thank_dest_path',
    'label' => $t('Thank You Page - Path'),
    'description' => $t('The URL path of the Thank You page.'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'default_value' => array(
      array(
        'value' => 'thank-you',
      ),
    ),
    'required' => TRUE,
  );
  $insts['citcon_thank_dest_title'] = array(
    'field_name' => 'citcon_thank_dest_title',
    'label' => $t('Thank You Page - Title'),
    'description' => $t('The title of the Thank You page.'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
  );
  $insts['citcon_thank_dest_text'] = array(
    'field_name' => 'citcon_thank_dest_text',
    'label' => $t('Thank You Page - Body'),
    'description' => $t('Thank you message to display after someone has filled out your form.'),
    'settings' => array(
      'text_processing' => 1,
    ),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  // "Thank you email" instances
  $insts['citcon_thank_email_status'] = array(
    'field_name' => 'citcon_thank_email_status',
    'label' => $t('Thank You Email'),
    'widget' => array(
      'type' => 'options_onoff',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_CHECKBOX_OFF,
      ),
    ),
  );
  $insts['citcon_thank_email_subject'] = array(
    'field_name' => 'citcon_thank_email_subject',
    'label' => $t('Thank You Email - Subject'),
    'description' => $t('Thank you message to display after someone has filled out your form.'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
  );
  $insts['citcon_thank_email_body'] = array(
    'field_name' => 'citcon_thank_email_body',
    'label' => $t('Thank You Email - Body'),
    'description' => $t('Thank you message to display after someone has filled out your form.'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );

  // "Thank you email" instances
  $insts['citcon_data_save_to_drupal'] = array(
    'field_name' => 'citcon_data_save_to_drupal',
    'label' => $t('Save data to Drupal'),
    'widget' => array(
      'type' => 'options_onoff',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_CHECKBOX_ON,
      ),
    ),
  );

  // "Data Export Plugin" instances... these will always be hidden.
  $insts['citcon_plugin_data_process'] = array(
    'field_name' => 'citcon_plugin_data_process',
    'label' => $t('Data Export Plugin'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );


  return $insts;
}

/**
 * Returns a structured array defining the fields created by the default content type.
 *
 * This is factored into this function so it can be used in both
 * citcon_install() and citcon_uninstall().
 *
 * @return
 *  An associative array specifying the fields we wish to add to our
 *  new node type.
 */
function _citcon_installed_supporter_fields() {
  // During installation, the t() function is unavailable, so we use get_t()
  // to store the name of the translation function.
  $t = get_t();

  // Initialize empty fields array
  $fields = array();

  //***********************
  // "CitCon Supporter" fields
  //***********************
  $fields['field_citcon_salutation'] = array(
    'field_name' => 'field_citcon_salutation',
    'cardinality' => 1,
    'type' => 'list_text',
    'settings' => array(
      'allowed_values' => array(
        'Mr' => $t("Mr"),
        'Mrs' => $t("Mrs"),
        'Ms' => $t("Ms"),
        'Dr' => $t("Dr"),
        'Rev' => $t("Rev"),
      ),
    ),
  );
  $fields['field_citcon_first_name'] = array(
    'field_name' => 'field_citcon_first_name',
    'cardinality' => 1,
    'type' => 'text',
  );
  $fields['field_citcon_middle_initial'] = array(
    'field_name' => 'field_citcon_middle_initial',
    'cardinality' => 1,
    'type' => 'text',
    'settings' => array(
      'max_length' => 1,
      'size' => 2,
    ),
  );
  $fields['field_citcon_last_name'] = array(
    'field_name' => 'field_citcon_last_name',
    'cardinality' => 1,
    'type' => 'text',
  );
  $fields['field_citcon_email'] = array(
    'field_name' => 'field_citcon_email',
    'cardinality' => 1,
    'type' => 'email',
  );
  $fields['field_citcon_address'] = array(
    'field_name' => 'field_citcon_address',
    'cardinality' => 1,
    'type' => 'text',
  );
  $fields['field_citcon_address_line2'] = array(
    'field_name' => 'field_citcon_address_line2',
    'cardinality' => 1,
    'type' => 'text',
  );
  $fields['field_citcon_city'] = array(
    'field_name' => 'field_citcon_city',
    'cardinality' => 1,
    'type' => 'text',
  );
  $fields['field_citcon_province'] = array(
    'field_name' => 'field_citcon_province',
    'cardinality' => 1,
    'type' => 'list_text',
    'settings' => array(
      'allowed_values' => array(
        "AB" => $t("Alberta"),
        "BC" => $t("British Columbia"),
        "MB" => $t("Manitoba"),
        "NB" => $t("New Brunswick"),
        "NL" => $t("Newfoundland and Labrador"),
        "NS" => $t("Nova Scotia"),
        "NT" => $t("Northwest Territories"),
        "NU" => $t("Nunavut"),
        "ON" => $t("Ontario"),
        "PE" => $t("Prince Edward Island"),
        "QC" => $t("Quebec"),
        "SK" => $t("Saskatchewan"),
        "YT" => $t("Yukon Territory"),
      ),
    ),
  );
  $fields['field_citcon_country'] = array(
    'field_name' => 'field_citcon_country',
    'cardinality' => 1,
    'type' => 'list_text',
    'settings' => array(
      'allowed_values' => array(
        "AF" => $t("Afghanistan"),
        "AL" => $t("Albania"),
        "DZ" => $t("Algeria"),
        "AS" => $t("American Samoa"),
        "AD" => $t("Andorra"),
        "AO" => $t("Angola"),
        "AI" => $t("Anguilla"),
        "AQ" => $t("Antarctica"),
        "AG" => $t("Antigua and Barbuda"),
        "AR" => $t("Argentina"),
        "AM" => $t("Armenia"),
        "AW" => $t("Aruba"),
        "AU" => $t("Australia"),
        "AT" => $t("Austria"),
        "AZ" => $t("Azerbaijan"),
        "BS" => $t("Bahamas"),
        "BH" => $t("Bahrain"),
        "BD" => $t("Bangladesh"),
        "BB" => $t("Barbados"),
        "BY" => $t("Belarus"),
        "BE" => $t("Belgium"),
        "BZ" => $t("Belize"),
        "BJ" => $t("Benin"),
        "BM" => $t("Bermuda"),
        "BT" => $t("Bhutan"),
        "BO" => $t("Bolivia"),
        "BA" => $t("Bosnia and Herzegovina"),
        "BW" => $t("Botswana"),
        "BV" => $t("Bouvet Island"),
        "BR" => $t("Brazil"),
        "IO" => $t("British Indian Ocean Territory"),
        "BN" => $t("Brunei Darussalam"),
        "BG" => $t("Bulgaria"),
        "BF" => $t("Burkina Faso"),
        "BI" => $t("Burundi"),
        "KH" => $t("Cambodia"),
        "CM" => $t("Cameroon"),
        "CA" => $t("Canada"),
        "CV" => $t("Cape Verde"),
        "KY" => $t("Cayman Islands"),
        "CF" => $t("Central African Republic"),
        "TD" => $t("Chad"),
        "CL" => $t("Chile"),
        "CN" => $t("China"),
        "CX" => $t("Christmas Island"),
        "CC" => $t("Cocos (Keeling) Islands"),
        "CO" => $t("Colombia"),
        "KM" => $t("Comoros"),
        "CG" => $t("Congo"),
        "CD" => $t("Congo, the Democratic Republic of the"),
        "CK" => $t("Cook Islands"),
        "CR" => $t("Costa Rica"),
        "CI" => $t("Cote D'Ivoire"),
        "HR" => $t("Croatia"),
        "CU" => $t("Cuba"),
        "CY" => $t("Cyprus"),
        "CZ" => $t("Czech Republic"),
        "DK" => $t("Denmark"),
        "DJ" => $t("Djibouti"),
        "DM" => $t("Dominica"),
        "DO" => $t("Dominican Republic"),
        "EC" => $t("Ecuador"),
        "EG" => $t("Egypt"),
        "SV" => $t("El Salvador"),
        "GQ" => $t("Equatorial Guinea"),
        "ER" => $t("Eritrea"),
        "EE" => $t("Estonia"),
        "ET" => $t("Ethiopia"),
        "FK" => $t("Falkland Islands (Malvinas)"),
        "FO" => $t("Faroe Islands"),
        "FJ" => $t("Fiji"),
        "FI" => $t("Finland"),
        "FR" => $t("France"),
        "GF" => $t("French Guiana"),
        "PF" => $t("French Polynesia"),
        "TF" => $t("French Southern Territories"),
        "GA" => $t("Gabon"),
        "GM" => $t("Gambia"),
        "GE" => $t("Georgia"),
        "DE" => $t("Germany"),
        "GH" => $t("Ghana"),
        "GI" => $t("Gibraltar"),
        "GR" => $t("Greece"),
        "GL" => $t("Greenland"),
        "GD" => $t("Grenada"),
        "GP" => $t("Guadeloupe"),
        "GU" => $t("Guam"),
        "GT" => $t("Guatemala"),
        "GN" => $t("Guinea"),
        "GW" => $t("Guinea-Bissau"),
        "GY" => $t("Guyana"),
        "HT" => $t("Haiti"),
        "HM" => $t("Heard Island and Mcdonald Islands"),
        "VA" => $t("Holy See (Vatican City State)"),
        "HN" => $t("Honduras"),
        "HK" => $t("Hong Kong"),
        "HU" => $t("Hungary"),
        "IS" => $t("Iceland"),
        "IN" => $t("India"),
        "ID" => $t("Indonesia"),
        "IR" => $t("Iran, Islamic Republic of"),
        "IQ" => $t("Iraq"),
        "IE" => $t("Ireland"),
        "IL" => $t("Israel"),
        "IT" => $t("Italy"),
        "JM" => $t("Jamaica"),
        "JP" => $t("Japan"),
        "JO" => $t("Jordan"),
        "KZ" => $t("Kazakhstan"),
        "KE" => $t("Kenya"),
        "KI" => $t("Kiribati"),
        "KP" => $t("Korea, Democratic People's Republic of"),
        "KR" => $t("Korea, Republic of"),
        "KW" => $t("Kuwait"),
        "KG" => $t("Kyrgyzstan"),
        "LA" => $t("Lao People's Democratic Republic"),
        "LV" => $t("Latvia"),
        "LB" => $t("Lebanon"),
        "LS" => $t("Lesotho"),
        "LR" => $t("Liberia"),
        "LY" => $t("Libyan Arab Jamahiriya"),
        "LI" => $t("Liechtenstein"),
        "LT" => $t("Lithuania"),
        "LU" => $t("Luxembourg"),
        "MO" => $t("Macao"),
        "MK" => $t("Macedonia, the Former Yugoslav Republic of"),
        "MG" => $t("Madagascar"),
        "MW" => $t("Malawi"),
        "MY" => $t("Malaysia"),
        "MV" => $t("Maldives"),
        "ML" => $t("Mali"),
        "MT" => $t("Malta"),
        "MH" => $t("Marshall Islands"),
        "MQ" => $t("Martinique"),
        "MR" => $t("Mauritania"),
        "MU" => $t("Mauritius"),
        "YT" => $t("Mayotte"),
        "MX" => $t("Mexico"),
        "FM" => $t("Micronesia, Federated States of"),
        "MD" => $t("Moldova, Republic of"),
        "MC" => $t("Monaco"),
        "MN" => $t("Mongolia"),
        "MS" => $t("Montserrat"),
        "MA" => $t("Morocco"),
        "MZ" => $t("Mozambique"),
        "MM" => $t("Myanmar"),
        "NA" => $t("Namibia"),
        "NR" => $t("Nauru"),
        "NP" => $t("Nepal"),
        "NL" => $t("Netherlands"),
        "AN" => $t("Netherlands Antilles"),
        "NC" => $t("New Caledonia"),
        "NZ" => $t("New Zealand"),
        "NI" => $t("Nicaragua"),
        "NE" => $t("Niger"),
        "NG" => $t("Nigeria"),
        "NU" => $t("Niue"),
        "NF" => $t("Norfolk Island"),
        "MP" => $t("Northern Mariana Islands"),
        "NO" => $t("Norway"),
        "OM" => $t("Oman"),
        "PK" => $t("Pakistan"),
        "PW" => $t("Palau"),
        "PS" => $t("Palestinian Territory, Occupied"),
        "PA" => $t("Panama"),
        "PG" => $t("Papua New Guinea"),
        "PY" => $t("Paraguay"),
        "PE" => $t("Peru"),
        "PH" => $t("Philippines"),
        "PN" => $t("Pitcairn"),
        "PL" => $t("Poland"),
        "PT" => $t("Portugal"),
        "PR" => $t("Puerto Rico"),
        "QA" => $t("Qatar"),
        "RE" => $t("Reunion"),
        "RO" => $t("Romania"),
        "RU" => $t("Russian Federation"),
        "RW" => $t("Rwanda"),
        "SH" => $t("Saint Helena"),
        "KN" => $t("Saint Kitts and Nevis"),
        "LC" => $t("Saint Lucia"),
        "PM" => $t("Saint Pierre and Miquelon"),
        "VC" => $t("Saint Vincent and the Grenadines"),
        "WS" => $t("Samoa"),
        "SM" => $t("San Marino"),
        "ST" => $t("Sao Tome and Principe"),
        "SA" => $t("Saudi Arabia"),
        "SN" => $t("Senegal"),
        "CS" => $t("Serbia and Montenegro"),
        "SC" => $t("Seychelles"),
        "SL" => $t("Sierra Leone"),
        "SG" => $t("Singapore"),
        "SK" => $t("Slovakia"),
        "SI" => $t("Slovenia"),
        "SB" => $t("Solomon Islands"),
        "SO" => $t("Somalia"),
        "ZA" => $t("South Africa"),
        "GS" => $t("South Georgia and the South Sandwich Islands"),
        "ES" => $t("Spain"),
        "LK" => $t("Sri Lanka"),
        "SD" => $t("Sudan"),
        "SR" => $t("Suriname"),
        "SJ" => $t("Svalbard and Jan Mayen"),
        "SZ" => $t("Swaziland"),
        "SE" => $t("Sweden"),
        "CH" => $t("Switzerland"),
        "SY" => $t("Syrian Arab Republic"),
        "TW" => $t("Taiwan, Province of China"),
        "TJ" => $t("Tajikistan"),
        "TZ" => $t("Tanzania, United Republic of"),
        "TH" => $t("Thailand"),
        "TL" => $t("Timor-Leste"),
        "TG" => $t("Togo"),
        "TK" => $t("Tokelau"),
        "TO" => $t("Tonga"),
        "TT" => $t("Trinidad and Tobago"),
        "TN" => $t("Tunisia"),
        "TR" => $t("Turkey"),
        "TM" => $t("Turkmenistan"),
        "TC" => $t("Turks and Caicos Islands"),
        "TV" => $t("Tuvalu"),
        "UG" => $t("Uganda"),
        "UA" => $t("Ukraine"),
        "AE" => $t("United Arab Emirates"),
        "GB" => $t("United Kingdom"),
        "US" => $t("United States"),
        "UM" => $t("United States Minor Outlying Islands"),
        "UY" => $t("Uruguay"),
        "UZ" => $t("Uzbekistan"),
        "VU" => $t("Vanuatu"),
        "VE" => $t("Venezuela"),
        "VN" => $t("Viet Nam"),
        "VG" => $t("Virgin Islands, British"),
        "VI" => $t("Virgin Islands, U.s."),
        "WF" => $t("Wallis and Futuna"),
        "EH" => $t("Western Sahara"),
        "YE" => $t("Yemen"),
        "ZM" => $t("Zambia"),
        "ZW" => $t("Zimbabwe"),
      ),
    ),
  );
  $fields['field_citcon_postal_code'] = array(
    'field_name' => 'field_citcon_postal_code',
    'cardinality' => 1,
    'type' => 'text',
  );
  $fields['field_citcon_mobile_num'] = array(
    'field_name' => 'field_citcon_mobile_num',
    'cardinality' => 1,
    'type' => 'text',
  );
  $fields['field_citcon_text_msg_opt_in'] = array(
    'field_name' => 'field_citcon_text_msg_opt_in',
    'cardinality' => 1,
    'type' => 'list_boolean',
    'settings' => array(
      'allowed_values' => array(
        CITCON_CHECKBOX_OFF => $t('Do not Opt In'),
        CITCON_CHECKBOX_ON => $t('Opt In'),
      ),
    ),
  );
  $fields['field_citcon_twitter_handle'] = array(
    'field_name' => 'field_citcon_twitter_handle',
    'cardinality' => 1,
    'type' => 'text',
  );
  $fields['field_citcon_comment'] = array(
    'field_name' => 'field_citcon_comment',
    'cardinality' => 1,
    'type' => 'text_long',
  );
  $fields['field_citcon_pet_opt_in'] = array(
    'field_name' => 'field_citcon_pet_opt_in',
    'cardinality' => 1,
    'type' => 'list_boolean',
    'settings' => array(
      'allowed_values' => array(
        CITCON_CHECKBOX_OFF => $t('Do not Display My Info In the Petition List'),
        CITCON_CHECKBOX_ON => $t('Display My Info In the Petition List'),
      ),
    ),
  );
  $fields['field_citcon_opt_in'] = array(
    'field_name' => 'field_citcon_opt_in',
    'cardinality' => 1,
    'type' => 'list_boolean',
    'settings' => array(
      'allowed_values' => array(
        CITCON_CHECKBOX_OFF => $t('Do not Opt In'),
        CITCON_CHECKBOX_ON => $t('Opt In'),
      ),
    ),
  );
  $fields['field_citcon_email_bcc_me'] = array(
    'field_name' => 'field_citcon_email_bcc_me',
    'cardinality' => 1,
    'type' => 'list_boolean',
    'settings' => array(
      'allowed_values' => array(
        CITCON_CHECKBOX_OFF => $t('Do not Send Myself a Copy of This Email'),
        CITCON_CHECKBOX_ON => $t('Send Myself a Copy of This Email'),
      ),
    ),
  );

  return $fields;
}

/**
 * Returns a structured array defining the instances for this content type.
 *
 * This is factored into this function so it can be used in both
 * node_example_install() and node_example_uninstall().
 *
 * @return
 *  An associative array specifying the instances we wish to add to our new
 *  node type.
 */
function _citcon_installed_supporter_instances() {
  $t = get_t();

  // Initialize empty instances array
  $insts = array();

  //***********************
  // "CitCon Supporter" instances
  //***********************
  $insts['field_citcon_salutation'] = array(
    'field_name' => 'field_citcon_salutation',
    'label' => $t('Salutation'),
    'widget' => array(
      'type' => 'options_select',
    ),
  );
  $insts['field_citcon_first_name'] = array(
    'field_name' => 'field_citcon_first_name',
    'label' => $t('First Name'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_middle_initial'] = array(
    'field_name' => 'field_citcon_middle_initial',
    'label' => $t('Middle Initial'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
  );
  $insts['field_citcon_last_name'] = array(
    'field_name' => 'field_citcon_last_name',
    'label' => $t('Last Name'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_email'] = array(
    'field_name' => 'field_citcon_email',
    'label' => $t('Email'),
    'widget' => array(
      'type' => 'email_textfield',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_address'] = array(
    'field_name' => 'field_citcon_address',
    'label' => $t('Address'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_address_line2'] = array(
    'field_name' => 'field_citcon_address_line2',
    'label' => $t('Address 2'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_city'] = array(
    'field_name' => 'field_citcon_city',
    'label' => $t('City'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_province'] = array(
    'field_name' => 'field_citcon_province',
    'label' => $t('Province'),
    'widget' => array(
      'type' => 'options_select',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_country'] = array(
    'field_name' => 'field_citcon_country',
    'label' => $t('Country'),
    'widget' => array(
      'type' => 'options_select',
    ),
    'default_value' => array(
      array(
        'value' => 'CA',
      ),
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_postal_code'] = array(
    'field_name' => 'field_citcon_postal_code',
    'label' => $t('Postal Code'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_mobile_num'] = array(
    'field_name' => 'field_citcon_mobile_num',
    'label' => $t('Mobile Phone Number'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_text_msg_opt_in'] = array(
    'field_name' => 'field_citcon_text_msg_opt_in',
    'label' => $t('Text Message Opt In'),
    'widget' => array(
      'type' => 'options_onoff',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_CHECKBOX_ON,
      ),
    ),
  );
  $insts['field_citcon_twitter_handle'] = array(
    'field_name' => 'field_citcon_twitter_handle',
    'label' => $t('Twitter Handle'),
    'widget' => array(
      'type' => 'text_textfield',
    ),
    'required' => TRUE,
  );
  $insts['field_citcon_comment'] = array(
    'field_name' => 'field_citcon_comment',
    'label' => $t('Comment'),
    'widget' => array(
      'type' => 'text_textarea',
    ),
  );
  $insts['field_citcon_pet_opt_in'] = array(
    'field_name' => 'field_citcon_pet_opt_in',
    'label' => $t('Petition Opt In'),
    'widget' => array(
      'type' => 'options_onoff',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_CHECKBOX_ON,
      ),
    ),
  );
  $insts['field_citcon_opt_in'] = array(
    'field_name' => 'field_citcon_opt_in',
    'label' => $t('Opt In'),
    'widget' => array(
      'type' => 'options_onoff',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_CHECKBOX_ON,
      ),
    ),
  );
  $insts['field_citcon_email_bcc_me'] = array(
    'field_name' => 'field_citcon_email_bcc_me',
    'label' => $t('Send Myself a Copy of This Email'),
    'widget' => array(
      'type' => 'options_onoff',
    ),
    'default_value' => array(
      array(
        'value' => CITCON_CHECKBOX_ON,
      ),
    ),
  );

  return $insts;
}

/**
 * Get a list of all the supporter types we'll be using
 */
function _citcon_installed_supporter_types() {
  $all_fields = _citcon_installed_supporter_fields();
  $types = array(
    'Citizens Connected Full' => array_keys($all_fields),
    'Email Full' => array(
      "field_citcon_salutation",
      "field_citcon_first_name",
      "field_citcon_middle_initial",
      "field_citcon_last_name",
      "field_citcon_email",
      "field_citcon_address",
      "field_citcon_address_line2",
      "field_citcon_city",
      "field_citcon_province",
      "field_citcon_country",
      "field_citcon_postal_code",
      "field_citcon_mobile_num",
      "field_citcon_text_msg_opt_in",
      "field_citcon_twitter_handle",
      "field_citcon_pet_opt_in",
      "field_citcon_opt_in",
    ),
    'Petition Full' => array(
      "field_citcon_salutation",
      "field_citcon_first_name",
      "field_citcon_middle_initial",
      "field_citcon_last_name",
      "field_citcon_email",
      "field_citcon_address",
      "field_citcon_address_line2",
      "field_citcon_city",
      "field_citcon_province",
      "field_citcon_country",
      "field_citcon_postal_code",
      "field_citcon_mobile_num",
      "field_citcon_text_msg_opt_in",
      "field_citcon_twitter_handle",
      "field_citcon_comment",
      "field_citcon_pet_opt_in",
      "field_citcon_opt_in",
    ),
    'Email Standard' => array(
      "field_citcon_first_name",
      "field_citcon_last_name",
      "field_citcon_email",
      "field_citcon_address",
      "field_citcon_city",
      "field_citcon_province",
      "field_citcon_country",
      "field_citcon_postal_code",
      "field_citcon_pet_opt_in",
      "field_citcon_opt_in",
    ),
    'Petition Standard' => array(
      "field_citcon_first_name",
      "field_citcon_last_name",
      "field_citcon_email",
      "field_citcon_address",
      "field_citcon_city",
      "field_citcon_province",
      "field_citcon_country",
      "field_citcon_postal_code",
      "field_citcon_comment",
      "field_citcon_pet_opt_in",
      "field_citcon_opt_in",
    ),
    'Email Standard Canada' => array(
      "field_citcon_first_name",
      "field_citcon_last_name",
      "field_citcon_email",
      "field_citcon_address",
      "field_citcon_city",
      "field_citcon_postal_code",
      "field_citcon_pet_opt_in",
      "field_citcon_opt_in",
    ),
    'Petition Standard Canada' => array(
      "field_citcon_first_name",
      "field_citcon_last_name",
      "field_citcon_email",
      "field_citcon_address",
      "field_citcon_city",
      "field_citcon_postal_code",
      "field_citcon_comment",
      "field_citcon_pet_opt_in",
      "field_citcon_opt_in",
    ),
    'Email Min Location' => array(
      "field_citcon_first_name",
      "field_citcon_email",
      "field_citcon_postal_code",
      "field_citcon_opt_in",
    ),
    'Petition Min Location' => array(
      "field_citcon_first_name",
      "field_citcon_email",
      "field_citcon_postal_code",
      "field_citcon_opt_in",
    ),
    'Email Min' => array(
      "field_citcon_first_name",
      "field_citcon_email",
      "field_citcon_opt_in",
    ),
    'Petition Min' => array(
      "field_citcon_first_name",
      "field_citcon_email",
      "field_citcon_opt_in",
    ),
  );
  return $types;
}
